// LWJGLProblem.java

import org.lwjgl.LWJGLException;
import org.lwjgl.opengl.AWTGLCanvas;
import org.lwjgl.opengl.PixelFormat;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import static org.lwjgl.opengl.GL11.*;

public class LWJGLProblem extends JFrame implements ChangeListener{
    private JSlider slider = new JSlider();
    private MyCanvas canvas;
    
    /**
     * Creates a new LWJGLProblem
     */
    public LWJGLProblem(){
        setSize(640, 480);
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setLayout(new BorderLayout());
        add(slider, BorderLayout.NORTH);
        
        try{
            canvas = new MyCanvas();
            add(canvas, BorderLayout.CENTER);
        }catch(LWJGLException e){
            e.printStackTrace();
        }
        slider.addChangeListener(this);
    }

    /**
     * Called when the slider moves
     * @param  e The event data
     */
    public void stateChanged(ChangeEvent e){
        canvas.repaint();
    }
    
    /**
     * Starts the program :)
     * @param  args The program arguments
     */
    public static void main(String[] args){
        LWJGLProblem frame = new LWJGLProblem();
        frame.setVisible(true);
    }
    
    /**
     * This is the AWTGLCanvas the will show the preview
     * @author  Tim Steenwyk
     */
    private class MyCanvas extends AWTGLCanvas{
        /**
         * Creates a new Canvas 
         * @throws  LWJGLException If we can't make a view
         */
        public MyCanvas() throws LWJGLException{
            super(new PixelFormat(8, 16, 0));
        }
        
        /**
         * Initializes OpenGL for the preview
         */
        public void initGL(){
            glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
            glDisable(GL_LIGHTING);
            glDisable(GL_DEPTH_TEST);
            glDepthMask(false);
            glEnable(GL_SCISSOR_TEST);
            glEnable(GL_BLEND);
            glEnable(GL_TEXTURE_2D);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
        }
        
        /**
         * Paints the preview
         */
        public void paintGL(){
            int width = getWidth();
            int height = getHeight();
            glClearColor((float)Math.random(), (float)Math.random(), (float)Math.random(), 1.0f);
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
            glViewport(0, 0, width, height);
            glScissor(0, 0, width, height);
            glMatrixMode(GL_PROJECTION);
            glLoadIdentity();
            glOrtho(0.0, width, height, 0.0, -1.0, 1.0);
            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();
            glColor4f(1.0f, 0.0f, 0.0f, 0.4f);

            glDisable(GL_TEXTURE_2D);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            glBegin(GL_QUADS);
                drawRectangle(5, 15, 108, 50);
                drawRectangle(15, 25, 108, 50);
                drawRectangle(25, 35, 108, 50);
                drawRectangle(35, 45, 108, 50);
                drawRectangle(45, 55, 108, 50);
            glEnd();
            
            if(glGetError() != GL_NO_ERROR){
                System.out.println("whoops");
            }
            
            try{
                swapBuffers();
            }catch(LWJGLException e){
                e.printStackTrace();
            }
        }
        
        /**
         * Draws a rectangle having the specified parameters 
         * @param  x The x location
         * @param  y The y location
         * @param  width The width of the rectangle
         * @param  height The height of the rectangle
         */
        private void drawRectangle(float x, float y, float width, float height){
            glVertex2f(x, y);
            glVertex2f(x, y + height);
            glVertex2f(x + width, y + height);
            glVertex2f(x + width, y);
        }
        
        /**
         * Handles errors that were thrown during a paint
         * @param  exception The exception that was thrown
         */
        protected void exceptionOccurred(LWJGLException exception){
            exception.printStackTrace();
        }
    }
}